/***
 * gtkMaggot - An improved Tron Game.
 * Copyright (C) 2003      Sylvain Rochet
 * Copyright (C) 2000-2002 Sebastien Bechet & Soazig Bars
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <stdlib.h>
#include <math.h>
#include <time.h>
#include <stdio.h>
#include <gtk/gtk.h>
#include <gdk/gdkkeysyms.h>
#include "game.h"
#include "flash.h"
#include "menu.h"
#include "main.h"
#include "sounds.h"

GSList *non_baveux;
joueur joueurs[MAXPLAYERS];
int nb_joueurs;
int maxpoints;

gint boucle_jeu_id = 0;

/* Gdk dependant */
void affiche_ecran(void) {

	gdk_draw_pixmap(darea->window, darea->style->fg_gc[GTK_WIDGET_STATE(darea)], pmap, 0, 0, 0, 0, WIDTH, HEIGHT);
}

/* Destruction des listes ... */
void free_non_baveux(void) {

	int i, lg;
	GSList *st;

	lg = g_slist_length(non_baveux);

	for(i = 0 ; i < lg ; i++) {

		st = g_slist_nth(non_baveux, i);
		ligne_kill((GSList*)st->data);
	}
	g_slist_free(non_baveux);
} 

/* Vidage/Nettoyage des listes ... */
void reset_jeu(gboolean reset_score) {

	int i,xb,yb;
	double tangle;

	if (reset_score) {

		joueurs[0].g = gdk_keyval_from_name("q");
		joueurs[0].d = gdk_keyval_from_name("a");
		joueurs[0].couleur=0;
		joueurs[1].g = gdk_keyval_from_name("s");
		joueurs[1].d = gdk_keyval_from_name("d");
		joueurs[1].couleur=1;
		joueurs[2].g = gdk_keyval_from_name("b");
		joueurs[2].d = gdk_keyval_from_name("n");
		joueurs[2].couleur=2;
		joueurs[3].g = GDK_Multi_key;
		joueurs[3].d = GDK_Menu;
		joueurs[3].couleur=3;
		joueurs[4].g = GDK_Down;
		joueurs[4].d = GDK_Right;
		joueurs[4].couleur=4;
		joueurs[5].g = GDK_KP_Add;
		joueurs[5].d = GDK_KP_Subtract;
		joueurs[5].couleur=5;
	}

	free_non_baveux();
	non_baveux = NULL;

	for(i = 0 ; i < MAXPLAYERS ; i++) {

		if (reset_score) joueurs[i].score = 0;

		joueurs[i].delay_score = 0;
		joueurs[i].mort = FALSE;
		joueurs[i].oldc.x = 36 + ( rand() % (WIDTH-2*36) );
		joueurs[i].oldc.y = 35 + ( rand() % (HEIGHT-2*35) );
		joueurs[i].iter = TPAS + ( rand() % (ITROU-TPAS) );
		joueurs[i].touche = rien;
	}

	for(i = 0, xb = 0, yb = 0 ; i < MAXPLAYERS ; i++) {

		xb += joueurs[i].oldc.x;
		yb += joueurs[i].oldc.y;
	}

	xb /= MAXPLAYERS;;
	yb /= MAXPLAYERS;

	for (i = 0 ; i< MAXPLAYERS ; i++) {

		if ( joueurs[i].oldc.x == xb ) 
			tangle=M_PI_2;
		else
			tangle = atan( ((double)joueurs[i].oldc.y-yb) / ((double)joueurs[i].oldc.x-xb) );

		if (joueurs[i].oldc.y>yb) {

			if (joueurs[i].oldc.x>xb)
				tangle = M_PI + tangle;
			else
				tangle = 2 * M_PI + tangle;

		} else {

			if (joueurs[i].oldc.x > xb)
				tangle = M_PI+tangle;
			else
				tangle = tangle;
		}
		joueurs[i].angle=((int)(0.5 + ( tangle * NANGLE / (2 * M_PI) ))) % NANGLE;
	}

	for (i = 0 ; i < MAXPLAYERS ; i++) {

		if (!active_players[i]) joueurs[i].mort=TRUE;
	}
}


static void rebond(int joueur) {

	int angle = joueurs[joueur].angle;

	/* up test */
	if (joueurs[joueur].coord.y <= 0) {

		if ( (angle > NANGLE / 2) && ( (angle > 7 * NANGLE / 8) || (angle < 5 * NANGLE / 8) ) ) {

			joueurs[joueur].angle = NANGLE - angle;
			sound_play(sound_bounce, FALSE);
		} else {

			joueurs[joueur].mort = TRUE;
			sound_play(sound_morts[joueur], FALSE);
			sound_play(sound_laugh, FALSE);
		}

	} else if (joueurs[joueur].coord.y >= HEIGHT - 2) {  /* bottom test */

		if ( (angle < NANGLE / 2) && ( (angle < NANGLE / 8) || (angle > 3 * NANGLE / 8) ) ) {

			joueurs[joueur].angle = NANGLE - angle;
			sound_play(sound_bounce, FALSE);
		} else {

			joueurs[joueur].mort = TRUE;
			sound_play(sound_morts[joueur], FALSE);
			sound_play(sound_laugh, FALSE);
		}

	} else if (joueurs[joueur].coord.x <= 0) {  /* left test */

		if ((angle < 3 * NANGLE / 8)  ||  (angle > 5 * NANGLE / 8)) {

			joueurs[joueur].angle = ((3 * NANGLE / 2) - angle) % NANGLE;
			sound_play(sound_bounce, FALSE);
		} else {

			joueurs[joueur].mort = TRUE;
			sound_play(sound_morts[joueur], FALSE);
			sound_play(sound_laugh, FALSE);
		}

	} else if (joueurs[joueur].coord.x >= WIDTH - 2) {  /* right test */

		if ((angle > NANGLE / 8)  &&  (angle < 7 * NANGLE / 8)) {

			joueurs[joueur].angle = (( 3 *NANGLE / 2) - angle) % NANGLE;
			sound_play(sound_bounce, FALSE);
		} else {

			joueurs[joueur].mort = TRUE;
			sound_play(sound_morts[joueur], FALSE);
			sound_play(sound_laugh, FALSE);
		}
	}
}

static GSList* comparaison(GSList *l, int noj) {

	int lg, lgdoub, lgnb;
	GSList *st, *stnb;
	int i, j, k;
	doublet *doub, *doubnb; 
	gboolean passe;
	int x,y,x2,y2;

	if (joueurs[noj].delay_score>0) joueurs[noj].delay_score--;

	x = joueurs[noj].oldc.x;
	y = joueurs[noj].oldc.y;;

	lg = g_slist_length(l);

	i = 0;
	while ( (i < lg)  &&  (!joueurs[noj].mort) ) {

		st = g_slist_nth(l,i);
		doub = (doublet*)st->data;
		x2 = doub->x;
		y2 = doub->y;

		/* died ? */ 
		if ((image_getpixel(doub->x  , doub->y  )!=noir) ||
			(image_getpixel(doub->x+1, doub->y  )!=noir) ||
			(image_getpixel(doub->x  , doub->y+1)!=noir) ||
			(image_getpixel(doub->x+1, doub->y+1)!=noir)) {

			if ( ((x2  == x  )&&(y2  == y  ))  ||
				((x2  == x+1)&&(y2  == y  ))  ||
				((x2  == x  )&&(y2  == y+1))  ||
				((x2  == x+1)&&(y2  == y+1))  ||
				((x2+1== x  )&&(y2  == y  ))  ||
				((x2+1== x+1)&&(y2  == y  ))  ||
				((x2+1== x  )&&(y2  == y+1))  ||
				((x2+1== x+1)&&(y2  == y+1))  ||
				((x2  == x  )&&(y2+1== y  ))  ||
				((x2  == x+1)&&(y2+1== y  ))  ||
				((x2  == x  )&&(y2+1== y+1))  ||
				((x2  == x+1)&&(y2+1== y+1))  ||
				((x2+1== x  )&&(y2+1== y  ))  ||
				((x2+1== x+1)&&(y2+1== y  ))  ||
				((x2+1== x  )&&(y2+1== y+1))  ||
				((x2+1== x+1)&&(y2+1== y+1)) ) {

			} else {

				joueurs[noj].mort = TRUE;
				sound_play(sound_morts[noj], FALSE);
				sound_play(sound_laugh, FALSE);

				for (j = i ; j < lg ; j++) {

					st = g_slist_last(l);
					doub = (doublet*)st->data;
					g_free(doub);
					l = g_slist_remove(l, (gpointer)doub);
				}
			}

		} else {

			/* win a point ? */
			lgnb = g_slist_length(non_baveux);

			for (j = 0 ; j < lgnb ; j++) {

				stnb = g_slist_nth(non_baveux, j);
				lgdoub = g_slist_length((GSList*)stnb->data);
				passe = FALSE;
				k = 0;

				while ((!passe) && (k<lgdoub)) {

					doubnb = (doublet*)(((GSList*)stnb->data)->data);

					if ( i != 0 ) {

						x = doubnb->x;
						y = doubnb->y;

						if ( ((x2  == x  )&&(y2  == y  ))||
							((x2  == x+1)&&(y2  == y  ))||
							((x2  == x  )&&(y2  == y+1))||
							((x2  == x+1)&&(y2  == y+1))||
							((x2+1== x  )&&(y2  == y  ))||
							((x2+1== x+1)&&(y2  == y  ))||
							((x2+1== x  )&&(y2  == y+1))||
							((x2+1== x+1)&&(y2  == y+1))||
							((x2  == x  )&&(y2+1== y  ))||
							((x2  == x+1)&&(y2+1== y  ))||
							((x2  == x  )&&(y2+1== y+1))||
							((x2  == x+1)&&(y2+1== y+1))||
							((x2+1== x  )&&(y2+1== y  ))||
							((x2+1== x+1)&&(y2+1== y  ))||
							((x2+1== x  )&&(y2+1== y+1))||
							((x2+1== x+1)&&(y2+1== y+1)) ) {

							if (joueurs[noj].delay_score == 0) {

								joueurs[noj].score++;
								sound_play(sound_thread,FALSE);
								joueurs[noj].delay_score = DSCORE;
								passe=TRUE;
							}
						}
					}
					k++;
				}
			}
		}
		i++;
	}

	return l;
}


gint boucle_jeu(void) {

	int i, j, x, y;
	GSList *l;

	for (i = 0 ; i < MAXPLAYERS ; i++) {

		if (!joueurs[i].mort) {

			switch (joueurs[i].touche) {

				case gauche:
					joueurs[i].angle--;
					break;
				case droit:
					joueurs[i].angle++;
					break;
				default:
					break;
			}

			/* Clipping of l'angle */
			if (joueurs[i].angle < 0) joueurs[i].angle += NANGLE;
			joueurs[i].angle %= NANGLE;

			x = joueurs[i].oldc.x + (0.5 + cos(joueurs[i].angle * 2 * M_PI / NANGLE) * TPAS);
			y = joueurs[i].oldc.y + (0.5 + sin(joueurs[i].angle * 2 * M_PI / NANGLE) * TPAS);


			/* Clipping of coords */
			if (y < 0) y = 0;
			if (y > HEIGHT) y = HEIGHT - 1;
			if (x < 0) x = 0;
			if (x > WIDTH) x = WIDTH - 1;

			joueurs[i].coord.x = x;
			joueurs[i].coord.y = y;

			/* test bounce */
			rebond(i);
    
			/* compute new step player */
			l = ligne(joueurs[i].oldc.x, joueurs[i].oldc.y, joueurs[i].coord.x, joueurs[i].coord.y);

			/* comparison with previous trace */
			l = comparaison(l, i);

			/* one least, enf game, +1 pts */
			if (joueurs[i].mort) {

				int nm;

				for (j = 0, nm = 0 ; j < MAXPLAYERS ; j++) {
					if (joueurs[j].mort) nm++;
				}

				if (nm == MAXPLAYERS) joueurs[i].score++;
			}

			if (l != 0) {

				if ( joueurs[i].iter >= (ITROU-TPAS) )
					non_baveux = g_slist_append(non_baveux,(gpointer)l);
				else {
					image_affpixels(l,joueurs[i].couleur);
					ligne_kill(l);
				}
			}

			joueurs[i].iter++;
			joueurs[i].iter %= ITROU;

			joueurs[i].oldc.x = joueurs[i].coord.x;
			joueurs[i].oldc.y = joueurs[i].coord.y;
		}
	}
	affiche_ecran();
	return 1;
}
