/***
 * gtkMaggot - An improved Tron Game.
 * Copyright (C) 2003      Sylvain Rochet
 * Copyright (C) 2000-2002 Sebastien Bechet & Soazig Bars
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <glib.h>
#include <gdk/gdk.h>
#include "image.h"


GdkColor colors_tab[14]={
	{0, 0xFFFF, 0x0000, 0x0000},
	{0, 0xFFFF, 0xFFFF, 0x0000},
	{0, 0xFFFF, 0x0000, 0xFFFF},
	{0, 0x0000, 0xFFFF, 0x0000},
	{0, 0x0000, 0xFFFF, 0xFFFF},
	{0, 0x0000, 0x0000, 0xFFFF},
	{0, 0x0000, 0x0000, 0x0000},
	{0, 0xFFFF, 0xFFFF, 0xFFFF},
	{0, 0x7FFF, 0x0000, 0x0000},
	{0, 0x7FFF, 0x7FFF, 0x0000},
	{0, 0x7FFF, 0x0000, 0x7FFF},
	{0, 0x0000, 0x7FFF, 0x0000},
	{0, 0x0000, 0x7FFF, 0x7FFF},
	{0, 0x0000, 0x0000, 0x7FFF}
};

#define TAILLE_TCOLOR (sizeof(colors_tab) / sizeof(GdkColor))

GdkPixmap *pmap;
GdkImage *image;
GdkVisual *visuel;
GdkColormap *cmap;
GdkGC **mgc;


int pixel2index(guint32 pixel) {

	int i;

	for (i = 0 ; i < TAILLE_TCOLOR ; i++) {

		if (colors_tab[i].pixel == pixel) return i;
	}
	return -1;
}


/* GDK Dependant */
void image_putpixel (int x,int y, int couleur) {

	gulong c;
	GdkGC *gc;

	c = colors_tab[couleur].pixel;
	gc = mgc[couleur];

	if ( (x <= image->width - 2)  &&  (x >= 0)  &&  (y <= image->height - 2)  &&  (y >= 0) ) {

		gdk_image_put_pixel(image, x, y, c);
		gdk_draw_point(pmap,gc,x,y);
		gdk_image_put_pixel(image, x+1, y, c);
		gdk_draw_point(pmap,gc,x+1,y);
		gdk_image_put_pixel(image, x, y+1, c);
		gdk_draw_point(pmap,gc,x,y+1);
		gdk_image_put_pixel(image, x+1, y+1, c);
		gdk_draw_point(pmap,gc,x+1,y+1);
	}
}

void image_putpixel_pm(int x,int y, int couleur) {

	gulong c;
	GdkGC *gc;

	c = colors_tab[couleur].pixel;
	gc = mgc[couleur];

	if ( (x <= image->width - 2)  &&  (x >= 0)  &&  (y <= image->height - 2)  &&  (y >= 0) ) {

		gdk_draw_point(pmap,gc,x,y);
		gdk_draw_point(pmap,gc,x+1,y);
		gdk_draw_point(pmap,gc,x,y+1);
		gdk_draw_point(pmap,gc,x+1,y+1);
	}
}

couleurs image_getpixel(int x,int y) {

	guint32 c;

	if ( (x >= 0)  &&  (x <= image->width - 1)  &&  (y >= 0)  &&  (y <= image->height - 1) )
		c = gdk_image_get_pixel(image, x, y);
	else
		c = noir;

	return pixel2index(c);
}

void image_affpixels(GSList *l, int couleur) {

	int i,lg;
	GSList *st;
	doublet *d;

	lg = g_slist_length(l);
	for (i = 0 ; i < lg ; i++) {

		st = g_slist_nth(l, i);
		d = (doublet*)st->data;
		image_putpixel(d->x, d->y, couleur);
	}
}


/* GDK dependant */
static void gc_init(void) {

	int i;
	GdkGCValues v;

	mgc = g_malloc(TAILLE_TCOLOR * sizeof(GdkGC*));

	for (i = 0 ; i < TAILLE_TCOLOR ; i++) {

		v.foreground = colors_tab[i];
		v.line_width = 1;
		mgc[i] = gdk_gc_new_with_values(pmap, &v, GDK_GC_FOREGROUND | GDK_GC_LINE_WIDTH);
	}
}

/* GDK dependant */
void image_reset(void) {

	int i, j;

	for (j = 0 ; j < image->height ; j++) {
		for (i = 0 ; i < image->width ; i++) {

			gdk_image_put_pixel(image,i,j,colors_tab[noir].pixel);
		}
	}

	gdk_draw_rectangle(pmap, mgc[noir], TRUE, 0, 0, image->width, image->height);
}

/* GDK dependant */
gboolean image_new(int largeur, int hauteur) {

	int i;
	visuel = gdk_visual_get_system();
	image = gdk_image_new(GDK_IMAGE_FASTEST,visuel,largeur,hauteur);
	cmap = gdk_colormap_get_system();
	pmap = gdk_pixmap_new(NULL, largeur, hauteur, visuel->depth);

	for (i = 0 ; i < TAILLE_TCOLOR ; i++) {

		if (!gdk_colormap_alloc_color(cmap, &colors_tab[i], FALSE, TRUE)) {

			g_error("Error when allocating colors");
		}
	}

	gc_init();
	image_reset();

	return TRUE;
}

void image_delete(void) {

	int i;

	gdk_image_destroy(image);
	gdk_pixmap_unref(pmap);
	gdk_visual_unref(visuel);
	gdk_colormap_unref(cmap);

	for (i = 0 ; i < TAILLE_TCOLOR ; i++) {

		gdk_gc_unref(mgc[i]);
	}
}

/*------------------------------------*/

void ligne_kill(GSList *l) {

	int i,lg;
	GSList *st;

	lg = g_slist_length(l);

	for (i = 0 ; i < lg ; i++) {

		st = g_slist_nth(l,i);
		g_free(st->data);
	}
	g_slist_free(l);
}

/* Implementation de l'algorithme de Bresenham */
GSList *ligne(int x, int y, int x2, int y2) {

	GSList *lt = NULL;
	doublet *p;
	int i, steep=0, sx, sy, dx, dy, e;

	dx = abs(x2 - x);
	sx = ((x2 - x) > 0) ? 1 : -1;
	dy = abs(y2 - y);
	sy = ((y2 - y) > 0) ? 1 : -1;

	if(dy > dx) {

		steep = 1;
		x ^= y;  /* swap(&x, &y) */
		y ^= x;
		x ^= y;
		dx ^= dy; /* swap(&dx, &dy) */
		dy ^= dx;
		dx ^= dy;
		sx ^= sy; /* swap(&sx,&sy) */
		sy ^= sx;
		sx ^= sy;
	}

	e = 2 * dy - dx;

	for(i = 0 ; i < dx ; i++) {

		if(steep) {

			p = g_malloc(sizeof(doublet));
			p->x = y;
			p->y = x;
			lt = g_slist_append(lt, (gpointer)p);
		} else {

			p = g_malloc(sizeof(doublet));
			p->x = x;
			p->y = y;
			lt = g_slist_append(lt, (gpointer)p);
		}

		while(e >= 0) {

			y += sy;
			e -= 2 * dx;
		}

		x += sx;
		e += 2 * dy;
	}

	p = g_malloc(sizeof(doublet));
	p->x = x2;
	p->y = y2;
	lt = g_slist_append(lt, (gpointer)p);

	return lt;
}
