/***
 * gtkMaggot - An improved Tron Game.
 * Copyright (C) 2003      Sylvain Rochet
 * Copyright (C) 2000-2002 Sebastien Bechet & Soazig Bars
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <stdio.h>
#include <time.h>
#include <stdlib.h>
#include <gdk/gdkkeysyms.h>
#include <gtk/gtk.h>
#include "game.h"
#include "main.h"
#include "menu.h"
#include "image.h"
#include "flash.h"
#include "sounds.h"

enum main_etats { menu, eclair, jeu, gameover, pause, sortie };

gboolean isBoardOk;
gboolean isPause;
gboolean isInGame;

GtkWidget *winmenu;
GtkWidget *window;
GtkWidget *darea;
GtkWidget *hbox, *vbox;
GtkWidget *lscores[MAXPLAYERS]; /* GtkLabel */
GtkWidget *gvs; /* GtkVSeparator */
gint timeout_id;
gint main_etat_old, main_etat;

gint wait_t = 40; /* 40 par defaut, 15 en mode suicide */

gint main_pause(void) {

	return TRUE;
}

gint main_gameover(void) {

	sound_play(sound_gameover,FALSE);
	gtk_timeout_remove(timeout_id);
	gtk_widget_hide(window);
	timeout_id = gtk_timeout_add(wait_t, (GtkFunction)main_menu, NULL);
	return FALSE;
}

gint main_jeu(void) {
	int i;
	int nm=0;
	gboolean fj = FALSE;
	char temp[8];

	isPause = FALSE;
	isInGame = TRUE;

	boucle_jeu();

	for (i = 0 , nm = 0 ; i < MAXPLAYERS ; i++) {

		if (joueurs[i].mort) nm++;
		if (joueurs[i].score >= maxpoints) fj=TRUE;
		sprintf(temp,"  %02d  ",joueurs[i].score);
		gtk_label_set_text(GTK_LABEL(lscores[i]),temp);
	}

	if (nm == MAXPLAYERS) {

		image_reset();
		sound_stop(sound_background);
		isInGame = FALSE;

		if (fj) {

			gtk_timeout_remove(timeout_id);
			timeout_id = gtk_timeout_add(wait_t, (GtkFunction)main_gameover, NULL);
			return FALSE;
		} else {

			reset_jeu(FALSE);
			gtk_timeout_remove(timeout_id);
			timeout_id = gtk_timeout_add(wait_t, (GtkFunction)main_flash, NULL);
			return FALSE;
		}
	}
	return TRUE;
}

gint main_flash(void) {

	static gint flash_compteur = 0;
	static gint flash_nojoueur = 0;

	while ( (! active_players[flash_nojoueur]) && (flash_nojoueur < MAXPLAYERS) ) {

		flash_nojoueur++;
	}
	if (flash_nojoueur > MAXPLAYERS - 1) flash_compteur = 25;

	if (flash_compteur >= 25) {

		if (flash_nojoueur >= MAXPLAYERS - 1) {

			gtk_timeout_remove(timeout_id);
			timeout_id = gtk_timeout_add(wait_t, (GtkFunction)main_jeu, NULL);
			sound_play(sound_background,TRUE);
			sound_play(sound_wohoo,FALSE);
			flash_compteur = 0;
			flash_nojoueur = 0;
			return FALSE;

		} else {

			image_putpixel_pm(joueurs[flash_nojoueur].oldc.x, joueurs[flash_nojoueur].oldc.y, joueurs[flash_nojoueur].couleur);
			flash_compteur=0;
			flash_nojoueur++;
			return TRUE;
		}
	}

	if (flash_compteur == 0) {

		sound_play(sound_start,FALSE);
	}

	flash_affiche(pmap, joueurs[flash_nojoueur].oldc.x - 33, joueurs[flash_nojoueur].oldc.y - 32, flash_compteur);
	affiche_ecran();

	flash_compteur++;

	return TRUE;
}

gint main_menu(void) {

	gtk_timeout_remove(timeout_id);
	gtk_widget_show(winmenu);
	return FALSE;
}

gboolean delete_event(GtkWidget *widget, GdkEvent *event, gpointer user_data) {

	gtk_main_quit();
	return FALSE;
}

gboolean clavier(GtkWidget *widget, GdkEventKey *event, gpointer user_data) {

	int i;

	if (!isInGame) return FALSE;

	if (event->type == GDK_KEY_RELEASE) {

		switch (event->keyval) {

			case GDK_Pause :
				if (!isPause) {

					sound_stop(sound_background);
					sound_play(sound_pause,FALSE);
					gtk_timeout_remove(timeout_id);
					timeout_id = gtk_timeout_add(wait_t, (GtkFunction)main_pause, NULL);
				} else {

					sound_play(sound_background,TRUE);
					gtk_timeout_remove(timeout_id);
					timeout_id = gtk_timeout_add(wait_t, (GtkFunction)main_jeu, NULL);
				}

				isPause = !isPause;
				return FALSE;

			case GDK_Escape :
				isInGame = FALSE;
				image_reset();
				sound_stop(sound_background);
				gtk_timeout_remove(timeout_id);
				timeout_id = gtk_timeout_add(wait_t, (GtkFunction)main_gameover, NULL);
				return FALSE;
		}
	}

	for (i = 0 ; i < MAXPLAYERS ; i++) {

		if (event->type == GDK_KEY_PRESS) {

			if (joueurs[i].g == event->keyval) {

				joueurs[i].touche = gauche;

			} else if (joueurs[i].d == event->keyval) {

				joueurs[i].touche=droit;
			}

		} else if (event->type==GDK_KEY_RELEASE) {

			if (joueurs[i].g==event->keyval) {

				if (joueurs[i].touche==gauche) joueurs[i].touche=rien;

			} else if (joueurs[i].d==event->keyval) {

				if (joueurs[i].touche==droit) joueurs[i].touche=rien;
			}

		}
	}
	return FALSE;
}

void main_new(void) {

	GtkStyle *gs;
	int i;

	window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
	gtk_window_set_policy(GTK_WINDOW(window), FALSE, FALSE, FALSE);

	gtk_window_set_title(GTK_WINDOW(window),"gtkMaggot");
	gtk_signal_connect(GTK_OBJECT(window), "delete_event", GTK_SIGNAL_FUNC(delete_event), NULL);

	darea = gtk_drawing_area_new();
	gtk_drawing_area_size(GTK_DRAWING_AREA(darea), WIDTH, HEIGHT);

	gtk_signal_connect(GTK_OBJECT(window), "key-press-event", GTK_SIGNAL_FUNC(clavier), NULL);
	gtk_signal_connect(GTK_OBJECT(window), "key-release-event", GTK_SIGNAL_FUNC(clavier), NULL);
	gtk_widget_set_events (window, GDK_KEY_PRESS_MASK|GDK_KEY_RELEASE_MASK);

	gs = gtk_widget_get_style(window);
	gs->bg[GTK_STATE_NORMAL] = colors_tab[noir];
	gs->bg[GTK_STATE_ACTIVE] = colors_tab[noir];
	gs->bg[GTK_STATE_PRELIGHT] = colors_tab[noir];
	gs->bg[GTK_STATE_SELECTED] = colors_tab[noir];
	gs->bg[GTK_STATE_INSENSITIVE] = colors_tab[noir];
	gtk_widget_set_style(window, gs);

	gs = gtk_widget_get_style(darea);
	gs->bg[GTK_STATE_NORMAL] = colors_tab[noir];
	gs->bg[GTK_STATE_ACTIVE] = colors_tab[noir];
	gs->bg[GTK_STATE_PRELIGHT] = colors_tab[noir];
	gs->bg[GTK_STATE_SELECTED] = colors_tab[noir];
	gs->bg[GTK_STATE_INSENSITIVE] = colors_tab[noir];
	gtk_widget_set_style(darea, gs);

	hbox = gtk_hbox_new(FALSE,0);
	vbox = gtk_vbox_new(TRUE,10);

	for (i = 0 ; i < MAXPLAYERS ; i++) {

		lscores[i] = gtk_label_new("  00  ");
		gs = gtk_style_copy(gtk_widget_get_style(lscores[i]));
		gs->fg[GTK_STATE_NORMAL] = colors_tab[i];
		gs->fg[GTK_STATE_ACTIVE] = colors_tab[i];
		gs->fg[GTK_STATE_PRELIGHT] = colors_tab[i];
		gs->fg[GTK_STATE_SELECTED] = colors_tab[i];
		gs->fg[GTK_STATE_INSENSITIVE] = colors_tab[i];

		gs->font = gdk_font_load("-adobe-helvetica-medium-r-normal--24-120-*-*-*-*-iso8859-1");

		gtk_widget_set_style(lscores[i],gs);
		gtk_widget_show(lscores[i]);
		gtk_box_pack_start_defaults(GTK_BOX(vbox), lscores[i]);
	}

	gtk_box_pack_start_defaults(GTK_BOX(hbox), darea);
	gvs = gtk_vseparator_new();

	gs = gtk_style_copy(gtk_widget_get_style(gvs));
	gs->bg[GTK_STATE_NORMAL] = colors_tab[blanc];
	gs->bg[GTK_STATE_ACTIVE] = colors_tab[blanc];
	gs->bg[GTK_STATE_PRELIGHT] = colors_tab[blanc];
	gs->bg[GTK_STATE_SELECTED] = colors_tab[blanc];
	gs->bg[GTK_STATE_INSENSITIVE] = colors_tab[blanc];
	gtk_widget_set_style(gvs,gs);

	gtk_box_pack_start_defaults(GTK_BOX(hbox), gvs);
	gtk_box_pack_start_defaults(GTK_BOX(hbox), vbox);

	gtk_container_add(GTK_CONTAINER(window), hbox);
  
	gtk_widget_show(gvs);
	gtk_widget_show(darea);
	gtk_widget_show(vbox);
	gtk_widget_show(hbox);
	 gtk_widget_show(window);
}

void main_delete(void) {

	if (isBoardOk) {
		free_non_baveux();
		flash_delete();
		image_delete();
	}
}

int main (int argc, char *argv[]) {

	srand(time(NULL));
	isBoardOk = FALSE;
	isInGame = FALSE;
	maxpoints = MAXPOINTS;

	gtk_init (&argc, &argv);

	sound_new();

	winmenu = win_menu();
	gtk_widget_show(winmenu);
	timeout_id = gtk_timeout_add(wait_t, (GtkFunction)main_menu, NULL);

	gtk_main();

	gtk_timeout_remove(timeout_id);
	main_delete();
	sound_delete();
 
	return TRUE;
}
