/***
 * gtkMaggot - An improved Tron Game.
 * Copyright (C) 2003      Sylvain Rochet
 * Copyright (C) 2000-2002 Sebastien Bechet & Soazig Bars
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <string.h>

#include <gdk/gdkkeysyms.h>
#include <gtk/gtk.h>

#include "../data/title.xpm"
#include "game.h"
#include "sounds.h"
#include "main.h"
#include "flash.h"
#include "menu.h"
#include "image.h"

gboolean active_players[MAXPLAYERS];

void on_j1_clicked(GtkButton *button, gpointer user_data) {

	sound_play(sound_b1,FALSE);
	active_players[0] = !active_players[0];
}
void on_j2_clicked(GtkButton *button, gpointer user_data) {

	sound_play(sound_b1,FALSE);
	active_players[1] = !active_players[1];
}
void on_j3_clicked(GtkButton *button, gpointer user_data) {

	sound_play(sound_b1,FALSE);
	active_players[2] = !active_players[2];
}
void on_j4_clicked(GtkButton *button, gpointer user_data) {

	sound_play(sound_b1,FALSE);
	active_players[3] = !active_players[3];
}
void on_j5_clicked(GtkButton *button, gpointer user_data) {

	sound_play(sound_b1,FALSE);
	active_players[4] = !active_players[4];
}
void on_j6_clicked(GtkButton *button, gpointer user_data) {

	sound_play(sound_b1,FALSE);
	active_players[5] = !active_players[5];
}

/* suicide mode */
void on_suicide_toggled(GtkToggleButton *togglebutton, gpointer user_data) {

	sound_play(sound_b4, FALSE);

	if (gtk_toggle_button_get_active(togglebutton)) {

		wait_t = 15;
	} else {

		wait_t = 40;
	}
}

/* Ok button */
void on_ok_clicked(GtkButton *button, gpointer user_data) {

	int i, j;

	for (i = 0, j = 0 ; i < MAXPLAYERS ; i++)
		if(active_players[i]) j++;

	sound_play(sound_b3,FALSE);

	if (j != 0) {

		sound_play(sound_load,FALSE);

		if (!isBoardOk) {

			main_new();
			image_new(WIDTH, HEIGHT);
			flash_new(window->window);
			isBoardOk = TRUE;
		}

		gtk_widget_hide(winmenu);
		reset_jeu(TRUE);
		gtk_widget_show(window);
		timeout_id = gtk_timeout_add(25, (GtkFunction)main_flash, NULL);
	}
	else  sound_play(sound_fail, FALSE);
}

/* Quit button */
void on_quit_clicked(GtkButton *button, gpointer user_data) {

	sound_play(sound_b3,FALSE);
	gtk_main_quit();
}

/* hscale Ajustment */
void on_adjust_changed(GtkAdjustment *adjust, gpointer user_data) {

	sound_play(sound_b2,FALSE);
	maxpoints = (int)adjust->value;
}

GtkWidget* win_menu(void) {

	GtkWidget *window1;
	GtkWidget *vbox1;
	GtkWidget *hbuttonbox1;
	GtkWidget *j1;
	GtkWidget *j2;
	GtkWidget *j3;
	GtkWidget *j4;
	GtkWidget *j5;
	GtkWidget *j6;
	GtkWidget *hbox2;
	GtkWidget *suicide;
	GtkWidget *vbox2;
	GtkWidget *label1;
	GtkWidget *hscale1;
	GtkObject *adjust;
	GtkWidget *hbox1;
	GtkWidget *pixmap1;
	GtkWidget *vbuttonbox1;
	GtkWidget *ok;
	GtkWidget *quit;

	GdkPixmap *gpixmap;
	GdkBitmap *gbitmap;
	GdkColormap *gcmap;

	GtkStyle *gs;

	int i;

	for(i = 0 ; i < MAXPLAYERS ; i++) active_players[i] = FALSE;

	window1 = gtk_window_new (GTK_WINDOW_TOPLEVEL);
	gtk_object_set_data (GTK_OBJECT (window1), "window1", window1);
	gtk_window_set_title (GTK_WINDOW (window1), "gtkMaggot Menu");
	gtk_signal_connect(GTK_OBJECT(window1), "delete_event", GTK_SIGNAL_FUNC(on_quit_clicked), NULL);

	vbox1 = gtk_vbox_new (FALSE, 0);
	gtk_widget_ref (vbox1);
	gtk_object_set_data_full (GTK_OBJECT (window1), "vbox1", vbox1, (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (vbox1);
	gtk_container_add (GTK_CONTAINER (window1), vbox1);
	gtk_container_set_border_width (GTK_CONTAINER (vbox1), 10);

	hbuttonbox1 = gtk_hbutton_box_new ();
	gtk_widget_ref (hbuttonbox1);
	gtk_object_set_data_full (GTK_OBJECT (window1), "hbuttonbox1", hbuttonbox1, (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (hbuttonbox1);
	gtk_box_pack_start (GTK_BOX (vbox1), hbuttonbox1, FALSE, FALSE, 0);
	gtk_button_box_set_spacing (GTK_BUTTON_BOX (hbuttonbox1), 0);
	gtk_button_box_set_child_size (GTK_BUTTON_BOX (hbuttonbox1), 64, 64);
	gtk_button_box_set_child_ipadding (GTK_BUTTON_BOX (hbuttonbox1), 0, 0);

	/* player 1 */
	j1 = gtk_toggle_button_new();

	gs = gtk_style_copy(gtk_widget_get_style(j1));
	gs->bg[GTK_STATE_NORMAL] = colors_tab[rougef];
	gs->bg[GTK_STATE_ACTIVE] = colors_tab[rouge];
	gs->bg[GTK_STATE_PRELIGHT] = colors_tab[rouge];
	gs->bg[GTK_STATE_SELECTED] = colors_tab[rouge];
	gs->bg[GTK_STATE_INSENSITIVE] = colors_tab[rougef];
	gtk_widget_set_style(j1,gs);

	gtk_widget_ref (j1);
	gtk_object_set_data_full (GTK_OBJECT (window1), "j1", j1, (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (j1);
	gtk_container_add (GTK_CONTAINER (hbuttonbox1), j1);
	GTK_WIDGET_SET_FLAGS (j1, GTK_CAN_DEFAULT);

	/* player 2 */
	j2 = gtk_toggle_button_new();
	gs = gtk_style_copy(gtk_widget_get_style(j2));
	gs->bg[GTK_STATE_NORMAL] = colors_tab[jaunef];
	gs->bg[GTK_STATE_ACTIVE] = colors_tab[jaune];
	gs->bg[GTK_STATE_PRELIGHT] = colors_tab[jaune];
	gs->bg[GTK_STATE_SELECTED] = colors_tab[jaune];
	gs->bg[GTK_STATE_INSENSITIVE] = colors_tab[jaunef];
	gtk_widget_set_style(j2,gs);
	gtk_widget_ref (j2);
	gtk_object_set_data_full (GTK_OBJECT (window1), "j2", j2, (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (j2);
	gtk_container_add (GTK_CONTAINER (hbuttonbox1), j2);
	GTK_WIDGET_SET_FLAGS (j2, GTK_CAN_DEFAULT);

	/* player 3 */
	j3 = gtk_toggle_button_new();
	gs = gtk_style_copy(gtk_widget_get_style(j3));
	gs->bg[GTK_STATE_NORMAL] = colors_tab[violetf];
	gs->bg[GTK_STATE_ACTIVE] = colors_tab[violet];
	gs->bg[GTK_STATE_PRELIGHT] = colors_tab[violet];
	gs->bg[GTK_STATE_SELECTED] = colors_tab[violet];
	gs->bg[GTK_STATE_INSENSITIVE] = colors_tab[violetf];
	gtk_widget_set_style(j3,gs);
	gtk_widget_ref (j3);
	gtk_object_set_data_full (GTK_OBJECT (window1), "j3", j3, (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (j3);
	gtk_container_add (GTK_CONTAINER (hbuttonbox1), j3);
	GTK_WIDGET_SET_FLAGS (j3, GTK_CAN_DEFAULT);

	/* player 4 */
	j4 = gtk_toggle_button_new();
	gtk_widget_ref (j4);
	gs = gtk_style_copy(gtk_widget_get_style(j4));
	gs->bg[GTK_STATE_NORMAL] = colors_tab[vertf];
	gs->bg[GTK_STATE_ACTIVE] = colors_tab[vert];
	gs->bg[GTK_STATE_PRELIGHT] = colors_tab[vert];
	gs->bg[GTK_STATE_SELECTED] = colors_tab[vert];
	gs->bg[GTK_STATE_INSENSITIVE] = colors_tab[vertf];
	gtk_widget_set_style(j4,gs);
	gtk_object_set_data_full (GTK_OBJECT (window1), "j4", j4, (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (j4);
	gtk_container_add (GTK_CONTAINER (hbuttonbox1), j4);
	GTK_WIDGET_SET_FLAGS (j4, GTK_CAN_DEFAULT);

	/* player 5 */
	j5 = gtk_toggle_button_new();
	gs = gtk_style_copy(gtk_widget_get_style(j5));
	gs->bg[GTK_STATE_NORMAL] = colors_tab[cyanf];
	gs->bg[GTK_STATE_ACTIVE] = colors_tab[cyan];
	gs->bg[GTK_STATE_PRELIGHT] = colors_tab[cyan];
	gs->bg[GTK_STATE_SELECTED] = colors_tab[cyan];
	gs->bg[GTK_STATE_INSENSITIVE] = colors_tab[cyanf];
	gtk_widget_set_style(j5,gs);
	gtk_widget_ref (j5);
	gtk_object_set_data_full (GTK_OBJECT (window1), "j5", j5, (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (j5);
	gtk_container_add (GTK_CONTAINER (hbuttonbox1), j5);
	GTK_WIDGET_SET_FLAGS (j5, GTK_CAN_DEFAULT);

	/* player 6 */
	j6 = gtk_toggle_button_new();
	gs = gtk_style_copy(gtk_widget_get_style(j6));
	gs->bg[GTK_STATE_NORMAL] = colors_tab[bleuf];
	gs->bg[GTK_STATE_ACTIVE] = colors_tab[bleu];
	gs->bg[GTK_STATE_PRELIGHT] = colors_tab[bleu];
	gs->bg[GTK_STATE_SELECTED] = colors_tab[bleu];
	gs->bg[GTK_STATE_INSENSITIVE] = colors_tab[bleu];
	gtk_widget_set_style(j6,gs);
	gtk_widget_ref (j6);
	gtk_object_set_data_full (GTK_OBJECT (window1), "j6", j6, (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (j6);
	gtk_container_add (GTK_CONTAINER (hbuttonbox1), j6);
	GTK_WIDGET_SET_FLAGS (j6, GTK_CAN_DEFAULT);



	hbox2 = gtk_hbox_new (FALSE, 0);
	gtk_widget_ref (hbox2);
	gtk_object_set_data_full (GTK_OBJECT (window1), "hbox2", hbox2, (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (hbox2);
	gtk_box_pack_start (GTK_BOX (vbox1), hbox2, TRUE, TRUE, 0);

	suicide = gtk_check_button_new_with_label ("Suicide mode");
	gtk_widget_ref (suicide);
	gtk_object_set_data_full (GTK_OBJECT (window1), "suicide", suicide, (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (suicide);
	gtk_box_pack_start (GTK_BOX (hbox2), suicide, FALSE, FALSE, 0);

	vbox2 = gtk_vbox_new (FALSE, 0);
	gtk_widget_ref (vbox2);
	gtk_object_set_data_full (GTK_OBJECT (window1), "vbox2", vbox2, (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (vbox2);
	gtk_box_pack_start (GTK_BOX (hbox2), vbox2, TRUE, TRUE, 92);

	label1 = gtk_label_new ("Score");
	gtk_widget_ref (label1);
	gtk_object_set_data_full (GTK_OBJECT (window1), "label1", label1, (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (label1);
	gtk_box_pack_start (GTK_BOX (vbox2), label1, FALSE, FALSE, 0);
	gtk_label_set_justify (GTK_LABEL (label1), GTK_JUSTIFY_LEFT);

	adjust = gtk_adjustment_new (maxpoints, 15, 99, 1, 10, 0);
	gtk_signal_connect(adjust, "value-changed", GTK_SIGNAL_FUNC(on_adjust_changed), NULL);

	hscale1 = gtk_hscale_new (GTK_ADJUSTMENT(adjust));
	gtk_widget_ref (hscale1);
	gtk_object_set_data_full (GTK_OBJECT (window1), "hscale1", hscale1, (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (hscale1);
	gtk_box_pack_start (GTK_BOX (vbox2), hscale1, TRUE, TRUE, 0);
	gtk_scale_set_value_pos (GTK_SCALE (hscale1), GTK_POS_BOTTOM);
	gtk_scale_set_digits (GTK_SCALE (hscale1), 0);

	hbox1 = gtk_hbox_new (FALSE, 0);
	gtk_widget_ref (hbox1);
	gtk_object_set_data_full (GTK_OBJECT (window1), "hbox1", hbox1, (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (hbox1);
	gtk_box_pack_start (GTK_BOX (vbox1), hbox1, TRUE, TRUE, 0);

	gcmap = gtk_widget_get_colormap (window1);
	gpixmap = gdk_pixmap_colormap_create_from_xpm_d(NULL, gcmap, &gbitmap, NULL, (gchar**)titre_xpm);
	pixmap1 = gtk_pixmap_new(gpixmap, gbitmap);
	gdk_pixmap_unref(gpixmap);
	gdk_bitmap_unref(gbitmap);
	gtk_widget_ref (pixmap1);
	gtk_object_set_data_full (GTK_OBJECT (window1), "pixmap1", pixmap1, (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (pixmap1);
	gtk_box_pack_start (GTK_BOX (hbox1), pixmap1, TRUE, TRUE, 0);

	vbuttonbox1 = gtk_vbutton_box_new ();
	gtk_widget_ref (vbuttonbox1);
	gtk_object_set_data_full (GTK_OBJECT (window1), "vbuttonbox1", vbuttonbox1, (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (vbuttonbox1);
	gtk_box_pack_start (GTK_BOX (hbox1), vbuttonbox1, TRUE, TRUE, 12);
	gtk_button_box_set_layout (GTK_BUTTON_BOX (vbuttonbox1), GTK_BUTTONBOX_SPREAD);
	gtk_button_box_set_spacing (GTK_BUTTON_BOX (vbuttonbox1), 0);
	gtk_button_box_set_child_size (GTK_BUTTON_BOX (vbuttonbox1), 80, 45);
	gtk_button_box_set_child_ipadding (GTK_BUTTON_BOX (vbuttonbox1), 6, 0);

	ok = gtk_button_new_with_label ("Go ! :-)");
	gtk_widget_ref (ok);
	gtk_object_set_data_full (GTK_OBJECT (window1), "ok", ok, (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (ok);
	gtk_container_add (GTK_CONTAINER (vbuttonbox1), ok);
	GTK_WIDGET_SET_FLAGS (ok, GTK_CAN_DEFAULT);

	quit = gtk_button_new_with_label ("Exit :-(");
	gtk_widget_ref (quit);
	gtk_object_set_data_full (GTK_OBJECT (window1), "quit", quit, (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (quit);
	gtk_container_add (GTK_CONTAINER (vbuttonbox1), quit);
	GTK_WIDGET_SET_FLAGS (quit, GTK_CAN_DEFAULT);

	gtk_signal_connect (GTK_OBJECT (j1), "clicked", GTK_SIGNAL_FUNC (on_j1_clicked), NULL);
	gtk_signal_connect (GTK_OBJECT (j2), "clicked", GTK_SIGNAL_FUNC (on_j2_clicked), NULL);
	gtk_signal_connect (GTK_OBJECT (j3), "clicked", GTK_SIGNAL_FUNC (on_j3_clicked), NULL);
	gtk_signal_connect (GTK_OBJECT (j4), "clicked", GTK_SIGNAL_FUNC (on_j4_clicked), NULL);
	gtk_signal_connect (GTK_OBJECT (j5), "clicked", GTK_SIGNAL_FUNC (on_j5_clicked), NULL);
	gtk_signal_connect (GTK_OBJECT (j6), "clicked", GTK_SIGNAL_FUNC (on_j6_clicked), NULL);
	gtk_signal_connect (GTK_OBJECT (suicide), "toggled", GTK_SIGNAL_FUNC (on_suicide_toggled), NULL);
	gtk_signal_connect (GTK_OBJECT (ok), "clicked", GTK_SIGNAL_FUNC (on_ok_clicked), NULL);
	gtk_signal_connect (GTK_OBJECT (quit), "clicked", GTK_SIGNAL_FUNC (on_quit_clicked), NULL);

	return window1;
}

